;
; This file contains a set of descriptors for a HID Interface.
;
#define	__EP0_BUFF_SIZE			  8	; 8, 16, 32, or 64
#define	__MAX_NUM_INT			  1	; For tracking Alternate Setting
#define	__MAX_EP_NUMBER			  1	; UEP1
#define	__NUM_CONFIGURATIONS	  1
#define	__NUM_INTERFACES		  1
#define	__MODE_PP				  _PPBM0
#define	__UCFG_VAL				  _PUEN | _TRINT | _FS | __MODE_PP	; Full-speed

#define __USB_USE_HID				; Inform the rest of the library subroutines that we're using a HID interface
;		
; Endpoints Allocation
;
#define	__HID_INTF_ID			  0
#define	__HID_UEP				  UEP1
#define	__HID_BD_OUT			  __ep1Bo
#define	__HID_INT_OUT_EP_SIZE	  64		; Output Buffer Size
#define	__HID_BD_IN				  __ep1Bi
#define	__HID_INT_IN_EP_SIZE	  64		; Input Buffer Size
#define	__HID_NUM_OF_DSC		  1

; ------------------------------------------------------------------------------
; This table is polled by the host immediately after USB Reset has been released.
; The table defines the maximum packet size EP0 can take.
; See section 9.6.1 of the Rev 1.0 USB specification.
; These fields are application dependent to be modified to meet your specifications.
;

__DeviceDescriptor
	Retlw	(__EndDeviceDescriptor - __DeviceDescriptor) / 2	; bLength Length of this descriptor
	Retlw	__DSC_DEV						; bDescType This is a DEVICE descriptor
	dt	0x10, 0x01							; bcdUSB Revision 1.10 low byte, high byte
	Retlw	0x00							; bDeviceClass zero means each interface operates independently
	Retlw	0x00							; bDeviceSubClass
	Retlw	0x00							; bDeviceProtocol
	Retlw	__EP0_BUFF_SIZE					; bMaxPacketSize for EP0

        ; idVendor low byte, high byte
	dt	0xD8, 0x04
        ; idProduct low byte, high byte
	dt	0x3C, 0x00

	dt	0x00, 0x00							; bcdDevice low byte, high byte
	Retlw	0x01							; iManufacturer
	Retlw	0x02							; iProduct

        ; iSerialNumber index
	Retlw	0x03

	Retlw	__NUM_CONFIGURATIONS 				        ; bNumConfigurations       ;Bootloader VID: 04D8 PID: 003C (VID = Microchip, PID = Microchip HID bootloader)
__EndDeviceDescriptor

; ------------------------------------------------------------------------------
; This table is retrieved by the host after the address has been set.
; This table defines the configurations available for the device.
; See section 9.6.2 of the Rev 1.0 USB specification (page 184).
; These fields are application dependent to be modified to meet your specifications.
;
; Configuration pointer table
__USB_CD_Ptr
__Configs
	db	Low ___Config1, High ___Config1
	db	Low ___Config1, High ___Config1

; Configuration Descriptor
___Config1
	Retlw	(__Interface1-___Config1) / 2	; bLength Length of this descriptor
	Retlw	__DSC_CFG						; bDescType 2 = CONFIGURATION
__Config1Len
	Retlw	Low ((__EndConfig1 - ___Config1) / 2)	; Length of this configuration
	Retlw	High ((__EndConfig1 - ___Config1) / 2)
	Retlw	0x01							; bNumInterfaces Number of interfaces
	Retlw	0x01							; bConfigValue Configuration Value
	Retlw	0x04							; iConfig String Index for this config = #01
	Retlw	0xA0							; bmAttributes attributes - bus powered

        ; Max power consumption (2X mA)
	Retlw	0x32
__Interface1
	Retlw	(__HIDDescriptor1 - __Interface1) / 2	; length of descriptor
	Retlw	__DSC_INTF
	Retlw	0x00							; number of interface, 0 based array
	Retlw	0x00							; alternate setting
	Retlw	0x02							; number of endpoints used in this interface
	Retlw	0x03							; interface CLASS - assigned by the USB
	Retlw	0x00							; boot device
	Retlw	0x00							; interface protocol - mouse
	Retlw 	0x00							; index to string descriptor that describes this interface
__HIDDescriptor1
	Retlw	(__Endpoint1In - __HIDDescriptor1) / 2	; descriptor size (9 bytes)
  	Retlw	0x21							; descriptor type (HID)
	dt		0x00, 0x00						; HID CLASS release number (1.00) low byte, high byte
   	Retlw	0x00							; Localised country code (none)
 	Retlw	0x01							; Number of HID CLASS descriptor to follow (1)
 	Retlw	0x22							; Report descriptor type (HID)
__ReportDescriptor1Len
	Retlw	Low ((__EndReportDescriptor1 - __ReportDescriptor1) / 2)
	Retlw	High ((__EndReportDescriptor1 - __ReportDescriptor1) / 2)
__Endpoint1In
	Retlw	(__EndPoint1Out - __Endpoint1In) / 2	; length of descriptor
	Retlw	__DSC_EP
	Retlw	0x81							; EP1, IN
	Retlw	0x03							; Interrupt
	dt	0x40, 0x00	    					; This should be the size of the endpoint buffer low byte, high byte
	Retlw	0x01							; Polling interval
__EndPoint1Out
	Retlw	(__EndConfig1 - __EndPoint1Out) / 2	; Length of this Endpoint Descriptor
	Retlw	__DSC_EP						; bDescriptorType = 5 for Endpoint Descriptor
	Retlw	0x01							; Endpoint number & direction
	Retlw	0x03							; Transfer type supported by this Endpoint
	dt	0x40, 0x00	    					; This should be the size of the endpoint buffer low byte, high byte
	Retlw	0x01							; Polling interval 
__EndConfig1

__ReportDescriptor1
    ; vendor defined usage page
    Retlw	0x06		
	Retlw	0x00       
    Retlw	0xFF

    ; vendor defined usage
    Retlw	0x09		
    Retlw	0x00

    ; collection(application)
    Retlw	0xA1		
    Retlw	0x01

    ; *** INPUT REPORT ***
               
    ; vendor defined usage
    Retlw	0x09		
    Retlw	0x01

    Retlw	0x15		; logical minimum (-128)
    Retlw	0x80        ; 
    Retlw	0x25		; logical maximum (127)
    Retlw	0x7F        ; 
    Retlw	0x35		; Physical Minimum (0)
    Retlw	0x00        ;
    Retlw	0x45		; Physical Maximum (255)
    Retlw	0xFF        ;
        
    ; report size in bits
    Retlw	0x75	
	Retlw	0x08
        
    ; report count (number of fields)
    Retlw	0x95		
	Retlw	0x40
        
    ; Input (Data, Variable, Absolute)
    Retlw	0x81		
	Retlw	0x02

    ; *** OUTPUT REPORT ***
    
    ; vendor defined usage
    Retlw	0x09		; usage (Vendor Defined)
	Retlw	0x02        ;

	Retlw	0x15		; logical minimum (-128)
	Retlw	0x80        ; 
    Retlw	0x25		; logical maximum (127)
	Retlw	0x7F        ; 
    Retlw	0x35		; Physical Minimum (0)
	Retlw	0x00        ;
    Retlw	0x45		; Physical Maximum (255)
	Retlw	0xFF        ;
        
    ; report size in bits
    Retlw	0x75		
	Retlw	0x08
        
    ; report count (number of fields)
    Retlw	0x95	
	Retlw	0x40
        
    ; Output (Data, Variable, Absolute)
    Retlw	0x91		
	Retlw	0x02

	Retlw   0xC0		; end collection
__EndReportDescriptor1

; String pointer table
__USB_SD_Ptr
__Strings
	db	Low __String0, High __String0
	db	Low __String1, High __String1
	db	Low __String2, High __String2
	db	Low __String3, High __String3
	db	Low __String4, High __String4

__String0
	Retlw	(__String1 - __String0) / 2		; Length of string 
	Retlw	__DSC_STR						; Descriptor type 3
	dt   0x09, 0x04    						; Language ID (0x0409 as defined by MS)

;company name
__String1
	Retlw	(__String2 - __String1) / 2
	Retlw	__DSC_STR
	
        Retlw   'J'
        Retlw   0x00
        Retlw   'G'
        Retlw   0x00
        Retlw   'B'
        Retlw   0x00
        Retlw   'T'
        Retlw   0x00
        Retlw   'o'
        Retlw   0x00
        Retlw   'o'
        Retlw   0x00
        Retlw   'l'
        Retlw   0x00
        Retlw   's'
        Retlw   0x00

;product name
__String2
	Retlw	(__EndString2 - __String2) / 2
	Retlw	__DSC_STR
	
        Retlw   'H'
        Retlw   0x00
        Retlw   'I'
        Retlw   0x00 
        Retlw   'D'
        Retlw   0x00
        Retlw   ' '
        Retlw   0x00
        Retlw   'B'
        Retlw   0x00
        Retlw   'o'
        Retlw   0x00
        Retlw   'o'
        Retlw   0x00
        Retlw   't'
        Retlw   0x00
        
__EndString2        

; serial number


__String3
	Retlw	(__String4 - __String3) / 2
	Retlw	__DSC_STR
        Retlw   'B'
        Retlw   0x00
        Retlw   'T'
        Retlw   0x00
        Retlw   '0'
        Retlw   0x00
        Retlw   '0'
        Retlw   0x00
        Retlw   '0'
        Retlw   0x00
        Retlw   '0'
        Retlw   0x00
        Retlw   '0'
        Retlw   0x00
        Retlw   '0'
        Retlw   0x00
        Retlw   '0'
        Retlw   0x00
        Retlw   '0'
        Retlw   0x00	
        Retlw   '0'
        Retlw   0x00
        Retlw   '0'
        Retlw   0x00
        Retlw   '0'
        Retlw   0x00
        Retlw   '0'
        Retlw   0x00
        Retlw   '0'
        Retlw   0x00
        Retlw   '1'
        Retlw   0x00
        
__String4
	Retlw	(__String5 - __String4) / 2
	Retlw	__DSC_STR
	dt 0x00,0x00
__String5
 
