    
    
   
     
;=== Compiler related defines ==================================
   
    Device 18F14K50
    
    Create_Coff = On
                                  
    Xtal = 24
	
    Dead_Code_Remove = On  
    Optimiser_Level = 3 
    Unsigned_Dwords = 1

    ' descriptor file, 
    USB_Descriptor "JGBBootLDescMC.inc" 	
    
;=== interrupt based defines ===============================

    On_Interrupt     GoTo USER_HIGH_INTERRUPT_VECTOR   ; RE-MAP INT VECTORS
    On_Low_Interrupt GoTo USER_LOW_INTERRUPT_VECTOR    ; RE-MAP INT VECTORS 
     
;=== Chip Fuse settings ============================================

Config_Start
   CPUDIV = CLKDIV2 ; CPU System Clock divided by 2
   USBDIV = OFF ; USB Clock comes directly from the OSC1/OSC2 oscillator block; no divide
   FOSC = HS ; HS oscillator
   PLLEN = On ; Oscillator multiplied by 4
   PCLKEN = OFF ; Primary clock disabled <<<<<<<<<<<<
   FCMEN = OFF ; Fail-Safe Clock Monitor disabled
   IESO = On ; Oscillator Switchover mode enabled     <<<<<<<<<<<<<<<<<<<<<
   PWRTEN = On ; PWRTEN enabled
   BOREN = On ; Brown-out Reset enabled and controlled by software (SBOREN is enabled)
   BORV = 27 ; VBOR set to 2.7 V nominal
   WDTEN = OFF ; WDT is controlled by SWDTEN bit of the WDTCON register
   MCLRE = OFF ; RE3 input pin enabled; MCLR disabled
   HFOFST = On ; HF-INTOSC starts clocking the CPU without waiting for the oscillator to stablize.
   STVREN = On ; Stack full/underflow will cause Reset
   LVP = OFF ; Low Voltage Programming disabled
   XINST = OFF ; Enhanced CPU disabled
   CP0 = OFF ; Block 0 not code-protected
   CP1 = OFF ; Block 1 not code-protected
   CPB = OFF ; Boot block not code-protected
   CPD = OFF ; Data EEPROM not code-protected
   WRT0 = OFF ; Block 0 not write-protected
   WRT1 = OFF ; Block 1 not write-protected
   WRTB = OFF ; Boot block not write-protected
   WRTC = OFF ; Configuration registers (300000-3000FFh) not write-protected
   WRTD = OFF ; Data EEPROM not write-protected
   EBTR0 = OFF ; Block 0 not protected from table reads executed in other blocks
   EBTR1 = OFF ; Block 1 not protected from table reads executed in other blocks
   EBTRB = OFF ; Boot block not protected from table reads executed in other blocks
Config_End



;=== SFR Alias's ===================================================


    Symbol NOT_RABPU = INTCON2.7    ' PORTA And PORTB Pull-up Enable Bit
  


    Symbol RABIF =  INTCON.0    ' RA And RB Port Change Interrupt Flag Bit(1)
    

;*=============================================================*
;* Description  Edata defines                                  *
;*=============================================================*

    Symbol FlashClearedStatus   = 255                                 ; Address in Eeeprom of the current status of the eeprom if cleared etc          
    
;*=============================================================*
;* Description  General Hardware I/0 defines                   *
;*=============================================================*

    Symbol USBDPlus = PORTA.0

   
;===============================================================

    Dim FSR_0 As FSR0L.Word

    
    ' General Variables
   
    Dim Index1 As Byte
    Dim Index2 As Byte
    Dim PromAddress As Word
    Dim MemoryPointer As Word
    Dim TempB1 As Byte
    Dim TempB2 As Byte

; Firmware related vars
    
    Dim BootOk As Bit

    Dim DelayUSBValue As Byte
    Dim DelayCount As Byte 


; Main Pic retated defines
   
    Symbol MainCodeStart            = $0B80                         ; Main code start address / Keep on a 64 byte boundry
    
    Symbol EndOfCodeSpace           = $4000                         ; End of code space
    Symbol FlashEraseSeg            = 64                            ; The size of memory bank erased in one go
    Symbol FlashWriteBlock          = 16                            ; The memory size Flash write needs to do in one block

       
;***************************************************************************
;   Command List
;   Below is the list of commands available and info on them
;
;  

    Symbol EraseProgram             = $01                           ; Erase memory
    ; Bytes expected 1 (command) 
    ; Returns $55 via USB when done
        
    Symbol WriteFlash               = $02                       
    
        
    Symbol FinishWrite             = $03                            ; Finish the write
    ; Bytes expected 1 (command) 
    ; Returns $55 via USB then and writes to eeprom to finish the write sequence 
    
    
    Symbol RunMainCode             = $04                            ; Run the main code above the bootloader
    ; Bytes expected 1 (command) 


    ' USB buffer...
    Symbol USBBufferSizeTX  = 64
    Symbol USBBufferSizeRX  = 64
    
    Dim USBInBuffer[USBBufferSizeRX] As Byte                        ; this is the receiving buffer (pic side) aliased to the USB RAM space    
    Dim USBOutBuffer[USBBufferSizeTX] As Byte
    

                
    Dim USBCommandIn As Byte At USBInBuffer#0
    Dim OffsetReadAlias As Byte At USBinBuffer#1
    Dim EepromAddressAlias As USBInBuffer#1
    Dim StartAddressAlias As Dword At USBInBuffer#1  
    Dim ByteCountAlias As Byte At USBInBuffer#5 
    Dim DataStartAlias As Byte At USBInBuffer#6
    
    ' some useful flags...
    Dim PP0 As Byte System        ' USBPOLL status return
    Symbol CARRY_FLAG = STATUS.0  ' high if microcontroller does not have control over the buffer
    Symbol ATTACHED_STATE = 6     ' is USB attached
    Symbol TRNIF = UIR.3		  ' low if USB Busy
          
 
    
     GoTo ProtonCodeStart

;===============================================================

   
' ************************************************************
' * program starts here...                                   *
' ************************************************************

ProtonCodeStart:
; Here we set up the pic as required and decide if were running USB Download mode or main code 
    
    OSCCON2.2 = 1                                                   ; Enable the Osc unit
    OSCCON = %01010000                                              ; ensure its on external osc

    All_Digital = 1
    
    ANSELH = 0                                                      ; Set up some analogue port settings
    ANSEL =  %00001000
      

    Clear                                                           ; Clear memory

    IOCA = %00000011                                                ; Set enable interrupts on port A to get the read on porta.0/1 to work
    
    NOT_RABPU = 0                                                   ; Enable pullups for port a and b
    WPUA = %00001000                                                ; Enable it for RA3 the button input
    WPUB = 0                                                        ; No pullups on port b 
            
    DelayMS 100                                                     ; A delay for all to settle


        
' ************************************************************
' * wait for USB interface to attach                         *
' ************************************************************

   GoSub IsBootOk                                                   ; Check if were ok to run now
   
   DelayMS 100                                         
   
   If USBDPlus = 0 Then GoTo USB_Enumerate
   If BootOk = 0 Then GoTo USB_Enumerate
   
   GoTo RunMain
                         

USB_Enumerate:
    IOCA = %00000000                                                ; Turn of the interrupts on PortA.0/1 thus disabling the read on the pins

    While 1 = 1
        Repeat												        ' \
        	USBPoll											        '   Wait for the USB interface to become attached
        Until PP0 = %00000110								        ' /
        GoTo USBMode
    Wend

    GoTo USBMode

RunMain:
    @ GoTo MainCodeStart                                            ; All is ok so run main code 
    
' ************************************************************
' * receive data from the USB bus                            *
' ************************************************************
DoUSBIn:
    Repeat
        USBIn 1, USBInBuffer, Auto
    Until STATUS.0 = 0
    Return
   
' ************************************************************
' * transmit Data                                            *
' ************************************************************
DoUSBOut:  

USBLoop: 
    USBPoll 
    If TRNIF = 1 Then GoTo USBLoop

    Repeat  
        USBOut 1, USBOutBuffer, USBBufferSizeTX                                
    Until STATUS.0 = 0     
         
    Return

    
' ************************************************************
'  Write the Data From the usbbuffer into flash ram

'  $20, address 4 bytes, payload 
'  
' ************************************************************   
   
WriteFlashData: 

    PromAddress = StartAddressAlias
    Index2 = ByteCountAlias - 1   
   
    If StartAddressAlias.Byte2 = $F0 Then GoTo WriteEepromValue     ; We do a jump to this routine and the code will return to the sender
    
    If PromAddress < MainCodeStart Then                             ; If the address were being requested to write to is not in the correct place then we just return                                                                                                                                       
        GoTo ResultOkExit
    EndIf
      
    FSR_0 = VarPtr DataStartAlias                                    ; point to the incoming data
    
    GoSub WriteToProm                                               ; Write that into the memory block and into Prom

   
' ************************************************************
'  Now Read the Data Back to confirm it was saved Ok
'  
' ************************************************************ 
    PromAddress = StartAddressAlias
    Index2 = ByteCountAlias
    FSR_0 = VarPtr DataStartAlias                                    ; point to the incoming data
       
    Index1 = 0
    Repeat                                                          ; Loop and test what's in memory with what is supposed to be there                                      
        TempB1 = CRead PromAddress
        TempB2 = POSTINC0
        If TempB1 <> TempB2 Then                                    ; If the data is not what was expected                                               
            GoTo SendBadResultandReturn                                ; Then send a fail result
        EndIf
        Inc PromAddress
        Inc Index1
    Until Index1 >= Index2

; If we get here data was written Ok so say so and return 

ResultOkExit:
    GoSub USBOK_Responce
    Return
    
SendBadResultandReturn:
    GoSub USBNotOK_Responce
    Return
    

; end of WriteFlashData sub
    
;==============================================================


' ************************************************************
'  Write over that Memory block the required data 
'  Then write that to Prom
' ************************************************************
WriteToProm:     

    MemoryPointer = PromAddress                                     ; Point to 16 byte boundary in memory were we want to load our data

    Index1 = 0
    Repeat
        TempB1 = POSTINC0
        CWrite MemoryPointer,[TempB1]  
        Inc Index1
        Inc MemoryPointer   
    Until Index1 >= FlashWriteBlock                                 ; keep writting until the full 16 byte block of ram is written to Prom
    
    Return
        

'' ************************************************************
''  System check to see if the last flash write went well
''  
'' ************************************************************ 
IsBootOk:            
    
    If ERead FlashClearedStatus = "W" Then                        ; Read in from eeprom the current write status 
        BootOk = 1        
    Else
        BootOk = 0                                  
    EndIf
    
    Return
    
' ************************************************************
'  Erase all of program memory above the boot loader line
'  
' ************************************************************    
EraseProgramData:

; First we write to eeprom we have started the write/clear phase
    
    EWrite FlashClearedStatus, ["C"]
    
    ; Now clear all of Flash above the bootloader
   
    PromAddress = MainCodeStart
    
    Repeat
        CErase PromAddress
        PromAddress = PromAddress + FlashEraseSeg
        USBPoll
    Until PromAddress >= EndOfCodeSpace 

    Clear USBOutBuffer

    GoSub USBOK_Responce
    Return 
    
' ************************************************************
'  Write to Eeprom the required values
'  
' ************************************************************  
WriteEepromValue:
    Index2 = USBInBuffer#5 - 1
    FSR_0 = VarPtr DataStartAlias
    
    TempB1 = EepromAddressAlias
    Index1 = 0
    Repeat
        EWrite TempB1,[POSTINC0]
        DelayUSBValue = 5
        GoSub DelayUSB
        Inc Index1 
        Inc TempB1
    Until Index1 = Index2
    GoSub USBOK_Responce
    Return 
        
' ************************************************************
'  A simple safe method of delaying and keeping the usb connection live
'  ' It will be longer than the required delay
' ************************************************************

DelayUSB:
    DelayCount = 0
    Repeat
        USBPoll
        DelayMS 1 
        Inc DelayCount
    Until DelayCount >= DelayUSBValue
    Return
                   
;_____________________________________________________________________________


; Send OK back on USB

USBOK_Responce:
    USBOutBuffer#0 = $55
    GoSub DoUSBOut
    Return

; Send back Not ok responce
USBNotOK_Responce:
    USBOutBuffer#0 = $AA
    GoSub DoUSBOut
    Return

;=== This is the main USB mode loop =========================

USBMode:
             
    While 1 = 1                                                     ; Main Loop Runs here
   
                    
        GoSub DoUSBIn
  
        Clear USBOutBuffer      
        
        Select USBCommandIn
                
            Case EraseProgram                                       ; Erase Program memory
                GoSub EraseProgramData                                              
                GoSub USBOK_Responce 
                
            Case WriteFlash                                         ;  Write Y bytes from Prom at X,X,                       
                GoSub WriteFlashData
                             
            Case FinishWrite                                        ; Set the appropriate flags to say it was a successful firmware update           
                EWrite FlashClearedStatus, ["W"]       
                GoSub USBOK_Responce 
                               
            Case RunMainCode                
                GoSub USBOK_Responce
                @ Call MainCodeStart                                ; Run the main code above the Bootloader
                

        EndSelect 
           
    Wend

    
Org  MainCodeStart + $8
USER_HIGH_INTERRUPT_VECTOR:
Org MainCodeStart + $18
USER_LOW_INTERRUPT_VECTOR:   


